==> 0.10.0.tar.gz: lib/src/services/db/db_service.dart <==
import 'dart:async';

import 'package:sqflite/sqflite.dart';

import '../../utils/log.dart';
import 'db_migration_handler.dart';

typedef Future<void> _OnCreateCallback(DbService db, int version);
typedef Future<void> _OnVersionChangeCallback(
    DbService db, int oldVersion, int newVersion);

/// Sqlite library, secondary packaging based on "sqflite/sqflite".
class DbService {
  static bool isLogEnable = false;

  late final String path;

  late final int currentVersion;
  late final _OnCreateCallback onCreate;
  late final _OnVersionChangeCallback onUpgrade;
  late final _OnVersionChangeCallback onDowngrade;

  late final Database _db;

  int _transactionCounter = 0;

  DbService({
    required this.path,
    required this.currentVersion,
    required this.onCreate,
    required this.onUpgrade,
    required this.onDowngrade,
  });

  DbService.wrapper(this._db);

  Future<void> init() async {
    if (isLogEnable) {
      Sqflite.devSetDebugModeOn(true);
    }

    _db = await openDatabase(
      path,
      version: currentVersion,
      onCreate: DbMigrationHandler.onCreate(this),
      onUpgrade: DbMigrationHandler.onUpgrade(this),
      onDowngrade: DbMigrationHandler.onDowngrade(this),
      onConfigure: (Database db) async {
        await db.execute("PRAGMA foreign_keys = ON");
      },
    );

    log('Database opened: $path');
  }

  void log(String str) {
    if (isLogEnable) Log.d(str);
  }

  /// Close the database. Cannot be accessed anymore
  Future<void> close() {
    return _db.close();
  }

  /// Get the database inner version
  Future<int> getVersion() {
    return _db.getVersion();
  }

  /// Automated transaction processing (lite)
  ///
  /// ```
  /// DbService db = Get.find();
  ///
  /// await db.transaction(() async {
  ///   await db.insert('table_name', ...);
  ///   await db.insert('table_name', ...);
  ///   await db.update('table_name', ...);
  ///   await deleteData();
  ///   await deleteData();
  /// });
  ///
  /// Future<void> deleteData() async {
  ///   await db.transaction(() async {
  ///     await db.delete('table_name', ...);
  ///     await db.delete('table_name', ...);
  ///   });
  /// }
  /// ```
  Future<void> transaction(Future<void> Function() actions) async {
    await begin();
    try {
      await actions();
      await commit();
    } catch (e) {
      await rollback();
      throw e;
    }
  }

  /// Automated transaction processing
  ///
  /// ```
  /// DbService db = Get.find();
  ///
  /// await db.begin();
  /// try {
  ///   await db.insert('table_name', ...);
  ///   await db.insert('table_name', ...);
  ///   await db.update('table_name', ...);
  ///   await deleteData();
  ///   await deleteData();
  ///   await db.commit();
  /// } catch (e) {
  ///   await db.rollback();
  ///   throw e;
  /// }
  ///
  /// Future<void> deleteData() async {
  /// await db.begin();
  /// try {
  ///   await db.delete('table_name', ...);
  ///   await db.delete('table_name', ...);
  ///   await db.commit();
  /// } catch (e) {
  ///   await db.rollback();
  ///   throw e;
  /// }
  /// }
  /// ```
  Future<void> begin() async {
    _transactionCounter++;
    if (_transactionCounter <= 1) {
      await _db.execute('BEGIN;');
    }
  }

  /// Transaction processing automation
  ///
  /// See the example code of [begin]
  Future<void> commit() async {
    _transactionCounter--;
    if (_transactionCounter <= 0) {
      await _db.execute("COMMIT;");
    }
  }

  /// Transaction processing automation
  ///
  /// See the example code of [begin]
  Future<void> rollback() async {
    if (_transactionCounter > 0) {
      await _db.execute('ROLLBACK;');
      _transactionCounter = 0;
    }
  }

  /// Execute an SQL query with no return value.
  ///
  /// ```
  ///   await db.execute(
  ///   'CREATE TABLE Test (id INTEGER PRIMARY KEY, name TEXT, value INTEGER, num REAL)');
  /// ```
  Future<void> execute(String sql, [List<Object?>? arguments]) {
    return _db.execute(sql, arguments);
  }

  /// Executes a raw SQL INSERT query and returns the last inserted row ID.
  ///
  /// ```
  /// int id1 = await database.rawInsert(
  ///   'INSERT INTO Test(name, value, num) VALUES("some name", 1234, 456.789)');
  /// ```
  ///
  /// 0 could be returned for some specific conflict algorithms if not inserted.
  Future<int> rawInsert(String sql, [List<Object?>? arguments]) {
    return _db.rawInsert(sql, arguments);
  }

  /// This method helps insert a map of [values]
  /// into the specified [table] and returns the
  /// id of the last inserted row.
  ///
  /// ```
  ///    var value = {
  ///      'age': 18,
  ///      'name': 'value'
  ///    };
  ///    int id = await db.insert(
  ///      'table',
  ///      value,
  ///      conflictAlgorithm: ConflictAlgorithm.replace,
  ///    );
  /// ```
  ///
  /// 0 could be returned for some specific conflict algorithms if not inserted.
  Future<int> insert(String table, Map<String, Object?> values,
      {String? nullColumnHack, ConflictAlgorithm? conflictAlgorithm}) {
    return _db.insert(table, values,
        nullColumnHack: nullColumnHack, conflictAlgorithm: conflictAlgorithm);
  }

  /// Determine whether there is data.
  ///
  /// [table] contains the table names to compile the query against.
  ///
  /// [where] filters which rows to return. Passing null will return all rows
  /// for the given URL. '?'s are replaced with the items in the
  /// [whereArgs] field.
  ///
  /// [groupBy] declares how to group rows. Passing null
  /// will cause the rows to not be grouped.
  ///
  /// [having] declares which row groups to include in the cursor,
  /// if row grouping is being used. Passing null will cause
  /// all row groups to be included, and is required when row
  /// grouping is not being used.
  ///
  /// ```
  ///  bool exists = await db.exists(tableTodo,
  ///      where: 'columnId = ?',
  ///      whereArgs: [id]);
  /// ```
  Future<bool> exists(String table,
      {String? where,
      List<Object?>? whereArgs,
      String? groupBy,
      String? having}) async {
    List res = await query(table,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        having: having,
        limit: 1);
    return res.isNotEmpty;
  }

  /// This is a helper to query a table and return the items found. All optional
  /// clauses and filters are formatted as SQL queries
  /// excluding the clauses' names.
  ///
  /// [table] contains the table names to compile the query against.
  ///
  /// [distinct] when set to true ensures each row is unique.
  ///
  /// The [columns] list specify which columns to return. Passing null will
  /// return all columns, which is discouraged.
  ///
  /// [where] filters which rows to return. Passing null will return all rows
  /// for the given URL. '?'s are replaced with the items in the
  /// [whereArgs] field.
  ///
  /// [groupBy] declares how to group rows. Passing null
  /// will cause the rows to not be grouped.
  ///
  /// [having] declares which row groups to include in the cursor,
  /// if row grouping is being used. Passing null will cause
  /// all row groups to be included, and is required when row
  /// grouping is not being used.
  ///
  /// [orderBy] declares how to order the rows,
  /// Passing null will use the default sort order,
  /// which may be unordered.
  ///
  /// [limit] limits the number of rows returned by the query.
  ///
  /// [offset] specifies the starting index.
  ///
  /// ```
  ///  List<Map> maps = await db.query(tableTodo,
  ///      columns: ['columnId', 'columnDone', 'columnTitle'],
  ///      where: 'columnId = ?',
  ///      whereArgs: [id]);
  /// ```
  Future<List<Map<String, Object?>>> query(String table,
      {bool? distinct,
      List<String>? columns,
      String? where,
      List<Object?>? whereArgs,
      String? groupBy,
      String? having,
      String? orderBy,
      int? limit,
      int? offset}) {
    return _db.query(table,
        distinct: distinct,
        columns: columns,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        having: having,
        orderBy: orderBy,
        limit: limit,
        offset: offset);
  }

  /// Executes a raw SQL SELECT query and returns a list
  /// of the rows that were found.
  ///
  /// ```
  /// List<Map> list = await database.rawQuery('SELECT * FROM Test');
  /// ```
  Future<List<Map<String, Object?>>> rawQuery(String sql,
      [List<Object?>? arguments]) {
    return _db.rawQuery(sql, arguments);
  }

  /// Executes a raw SQL UPDATE query and returns
  /// the number of changes made.
  ///
  /// ```
  /// int count = await database.rawUpdate(
  ///   'UPDATE Test SET name = ?, value = ? WHERE name = ?',
  ///   ['updated name', '9876', 'some name']);
  /// ```
  Future<int> rawUpdate(String sql, [List<Object?>? arguments]) {
    return _db.rawUpdate(sql, arguments);
  }

  /// Convenience method for updating rows in the database. Returns
  /// the number of changes made
  ///
  /// Update [table] with [values], a map from column names to new column
  /// values. null is a valid value that will be translated to NULL.
  ///
  /// [where] is the optional WHERE clause to apply when updating.
  /// Passing null will update all rows.
  ///
  /// You may include ?s in the where clause, which will be replaced by the
  /// values from [whereArgs]
  ///
  /// [conflictAlgorithm] (optional) specifies algorithm to use in case of a
  /// conflict. See [ConflictAlgorithm] docs for more details
  ///
  /// ```
  /// int count = await db.update(tableTodo, todo.toMap(),
  ///    where: '$columnId = ?', whereArgs: [todo.id]);
  /// ```
  Future<int> update(String table, Map<String, Object?> values,
      {String? where,
      List<Object?>? whereArgs,
      ConflictAlgorithm? conflictAlgorithm}) {
    return _db.update(table, values,
        where: where,
        whereArgs: whereArgs,
        conflictAlgorithm: conflictAlgorithm);
  }

  /// Executes a raw SQL DELETE query and returns the
  /// number of changes made.
  ///
  /// ```
  /// int count = await database
  ///   .rawDelete('DELETE FROM Test WHERE name = ?', ['another name']);
  /// ```
  Future<int> rawDelete(String sql, [List<Object?>? arguments]) {
    return _db.rawDelete(sql, arguments);
  }

  /// Convenience method for deleting rows in the database.
  ///
  /// Delete from [table]
  ///
  /// [where] is the optional WHERE clause to apply when updating. Passing null
  /// will delete all rows.
  ///
  /// You may include ?s in the where clause, which will be replaced by the
  /// values from [whereArgs]
  ///
  /// Returns the number of rows affected.
  /// ```
  ///  int count = await db.delete(tableTodo, where: 'columnId = ?', whereArgs: [id]);
  /// ```
  Future<int> delete(String table, {String? where, List<Object?>? whereArgs}) {
    return _db.delete(table, where: where, whereArgs: whereArgs);
  }
}

==> 2.0.0.tar.gz: lib/src/services/db/db_service.dart <==
import 'dart:async';

import 'package:sqflite/sqflite.dart';

import '../../utils/log.dart';
import 'db_migration_handler.dart';

typedef Future<void> _OnCreateCallback(DbService db, int version);
typedef Future<void> _OnVersionChangeCallback(
    DbService db, int oldVersion, int newVersion);

/// Sqlite library, secondary packaging based on "sqflite/sqflite".
class DbService {
  static bool isLogEnable = false;

  late final String path;

  late final int currentVersion;
  late final _OnCreateCallback onCreate;
  late final _OnVersionChangeCallback onUpgrade;
  late final _OnVersionChangeCallback onDowngrade;

  late final Database _db;

  int _transactionCounter = 0;

  DbService({
    required this.path,
    required this.currentVersion,
    required this.onCreate,
    required this.onUpgrade,
    required this.onDowngrade,
  });

  DbService.wrapper(this._db);

  Future<void> init() async {
    if (isLogEnable) {
      Sqflite.devSetDebugModeOn(true);
    }

    _db = await openDatabase(
      path,
      version: currentVersion,
      onCreate: DbMigrationHandler.onCreate(this),
      onUpgrade: DbMigrationHandler.onUpgrade(this),
      onDowngrade: DbMigrationHandler.onDowngrade(this),
      onConfigure: (Database db) async {
        await db.execute("PRAGMA foreign_keys = ON");
      },
    );

    log('Database opened: $path');
  }

  void log(String str) {
    if (isLogEnable) Log.d(str);
  }

  /// Close the database. Cannot be accessed anymore
  Future<void> close() {
    return _db.close();
  }

  /// Get the database inner version
  Future<int> getVersion() {
    return _db.getVersion();
  }

  /// Automated transaction processing (lite)
  ///
  /// ```
  /// DbService db = Get.find();
  ///
  /// await db.transaction(() async {
  ///   await db.insert('table_name', ...);
  ///   await db.insert('table_name', ...);
  ///   await db.update('table_name', ...);
  ///   await deleteData();
  ///   await deleteData();
  /// });
  ///
  /// Future<void> deleteData() async {
  ///   await db.transaction(() async {
  ///     await db.delete('table_name', ...);
  ///     await db.delete('table_name', ...);
  ///   });
  /// }
  /// ```
  Future<T> transaction<T>(Future<T> Function() actions) async {
    await begin();
    try {
      var res = await actions();
      await commit();

      return res;
    } catch (e) {
      await rollback();
      throw e;
    }
  }

  /// Automated transaction processing
  ///
  /// ```
  /// DbService db = Get.find();
  ///
  /// await db.begin();
  /// try {
  ///   await db.insert('table_name', ...);
  ///   await db.insert('table_name', ...);
  ///   await db.update('table_name', ...);
  ///   await deleteData();
  ///   await deleteData();
  ///   await db.commit();
  /// } catch (e) {
  ///   await db.rollback();
  ///   throw e;
  /// }
  ///
  /// Future<void> deleteData() async {
  ///   await db.begin();
  ///   try {
  ///     await db.delete('table_name', ...);
  ///     await db.delete('table_name', ...);
  ///     await db.commit();
  ///   } catch (e) {
  ///     await db.rollback();
  ///     throw e;
  ///   }
  /// }
  /// ```
  Future<void> begin() async {
    _transactionCounter++;
    if (_transactionCounter <= 1) {
      await _db.execute('BEGIN;');
    }
  }

  /// Transaction processing automation
  ///
  /// See the example code of [begin]
  Future<void> commit() async {
    _transactionCounter--;
    if (_transactionCounter <= 0) {
      await _db.execute("COMMIT;");
    }
  }

  /// Transaction processing automation
  ///
  /// See the example code of [begin]
  Future<void> rollback() async {
    if (_transactionCounter > 0) {
      await _db.execute('ROLLBACK;');
      _transactionCounter = 0;
    }
  }

  /// Execute an SQL query with no return value.
  ///
  /// ```
  ///   await db.execute(
  ///   'CREATE TABLE Test (id INTEGER PRIMARY KEY, name TEXT, value INTEGER, num REAL)');
  /// ```
  Future<void> execute(String sql, [List<Object?>? arguments]) {
    return _db.execute(sql, arguments);
  }

  /// Executes a raw SQL INSERT query and returns the last inserted row ID.
  ///
  /// ```
  /// int id1 = await database.rawInsert(
  ///   'INSERT INTO Test(name, value, num) VALUES("some name", 1234, 456.789)');
  /// ```
  ///
  /// 0 could be returned for some specific conflict algorithms if not inserted.
  Future<int> rawInsert(String sql, [List<Object?>? arguments]) {
    return _db.rawInsert(sql, arguments);
  }

  /// This method helps insert a map of [values]
  /// into the specified [table] and returns the
  /// id of the last inserted row.
  ///
  /// ```
  ///    var value = {
  ///      'age': 18,
  ///      'name': 'value'
  ///    };
  ///    int id = await db.insert(
  ///      'table',
  ///      value,
  ///      conflictAlgorithm: ConflictAlgorithm.replace,
  ///    );
  /// ```
  ///
  /// 0 could be returned for some specific conflict algorithms if not inserted.
  Future<int> insert(String table, Map<String, Object?> values,
      {String? nullColumnHack, ConflictAlgorithm? conflictAlgorithm}) {
    return _db.insert(table, values,
        nullColumnHack: nullColumnHack, conflictAlgorithm: conflictAlgorithm);
  }

  /// Determine whether there is data.
  ///
  /// [table] contains the table names to compile the query against.
  ///
  /// [where] filters which rows to return. Passing null will return all rows
  /// for the given URL. '?'s are replaced with the items in the
  /// [whereArgs] field.
  ///
  /// ```
  ///  bool exists = await db.exists('users',
  ///      where: 'username = ?',
  ///      whereArgs: ['jobs']);
  /// ```
  Future<bool> exists(
    String table, {
    String? where,
    List<Object?>? whereArgs,
  }) async {
    return (await query(
      table,
      where: where,
      whereArgs: whereArgs,
      limit: 1,
    ))
        .isNotEmpty;
  }

  /// This is a helper to query a table and return the first data.
  /// If no data is found, it returns "null".
  /// All optional clauses and filters are formatted as SQL queries excluding the clauses' names.
  ///
  /// [table] contains the table names to compile the query against.
  ///
  /// [distinct] when set to true ensures each row is unique.
  ///
  /// The [columns] list specify which columns to return. Passing null will
  /// return all columns, which is discouraged.
  ///
  /// [where] filters which rows to return. Passing null will return all rows
  /// for the given URL. '?'s are replaced with the items in the
  /// [whereArgs] field.
  ///
  /// [groupBy] declares how to group rows. Passing null
  /// will cause the rows to not be grouped.
  ///
  /// [having] declares which row groups to include in the cursor,
  /// if row grouping is being used. Passing null will cause
  /// all row groups to be included, and is required when row
  /// grouping is not being used.
  ///
  /// [orderBy] declares how to order the rows,
  /// Passing null will use the default sort order,
  /// which may be unordered.
  ///
  /// ```
  ///  Map? userData = await db.queryOne('users',
  ///      where: 'username = ?',
  ///      whereArgs: ['jobs']);
  /// ```
  Future<Map<String, Object?>?> queryOne(String table,
      {bool? distinct,
      List<String>? columns,
      String? where,
      List<Object?>? whereArgs,
      String? groupBy,
      String? having,
      String? orderBy}) async {
    var rows = await _db.query(table,
        distinct: distinct,
        columns: columns,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        having: having,
        orderBy: orderBy,
        limit: 1);

    return rows.isNotEmpty ? rows[0] : null;
  }

  /// This is a helper to query a table and assembles the found items into a Map with [keyField] as the key.
  /// All optional clauses and filters are formatted as SQL queries excluding the clauses' names.
  ///
  /// [T] indicates the type of [keyField] field
  ///
  /// [table] contains the table names to compile the query against.
  ///
  /// [distinct] when set to true ensures each row is unique.
  ///
  /// The [columns] list specify which columns to return. Passing null will
  /// return all columns, which is discouraged.
  ///
  /// [where] filters which rows to return. Passing null will return all rows
  /// for the given URL. '?'s are replaced with the items in the
  /// [whereArgs] field.
  ///
  /// [groupBy] declares how to group rows. Passing null
  /// will cause the rows to not be grouped.
  ///
  /// [having] declares which row groups to include in the cursor,
  /// if row grouping is being used. Passing null will cause
  /// all row groups to be included, and is required when row
  /// grouping is not being used.
  ///
  /// [orderBy] declares how to order the rows,
  /// Passing null will use the default sort order,
  /// which may be unordered.
  ///
  /// [limit] limits the number of rows returned by the query.
  ///
  /// [offset] specifies the starting index.
  ///
  /// ```
  ///  Map<String, Map> maps = await db.queryMap<String>('users', keyField: 'username');
  /// ```
  Future<Map<T, Map<String, Object?>>> queryMap<T>(String table,
      {required String keyField,
      bool? distinct,
      List<String>? columns,
      String? where,
      List<Object?>? whereArgs,
      String? groupBy,
      String? having,
      String? orderBy,
      int? limit,
      int? offset}) async {
    Map<T, Map<String, Object?>> res = {};

    var rows = await _db.query(table,
        distinct: distinct,
        columns: columns,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        having: having,
        orderBy: orderBy,
        limit: limit,
        offset: offset);

    rows.forEach((e) {
      if (e.containsKey(keyField) && e[keyField] is T) {
        res[e[keyField] as T] = e;
      }
    });

    return res;
  }

  /// This is a helper to query a table and return the items found. All optional
  /// clauses and filters are formatted as SQL queries
  /// excluding the clauses' names.
  ///
  /// [table] contains the table names to compile the query against.
  ///
  /// [distinct] when set to true ensures each row is unique.
  ///
  /// The [columns] list specify which columns to return. Passing null will
  /// return all columns, which is discouraged.
  ///
  /// [where] filters which rows to return. Passing null will return all rows
  /// for the given URL. '?'s are replaced with the items in the
  /// [whereArgs] field.
  ///
  /// [groupBy] declares how to group rows. Passing null
  /// will cause the rows to not be grouped.
  ///
  /// [having] declares which row groups to include in the cursor,
  /// if row grouping is being used. Passing null will cause
  /// all row groups to be included, and is required when row
  /// grouping is not being used.
  ///
  /// [orderBy] declares how to order the rows,
  /// Passing null will use the default sort order,
  /// which may be unordered.
  ///
  /// [limit] limits the number of rows returned by the query.
  ///
  /// [offset] specifies the starting index.
  ///
  /// ```
  ///  List<Map> maps = await db.query(tableTodo,
  ///      columns: ['columnId', 'columnDone', 'columnTitle'],
  ///      where: 'columnId = ?',
  ///      whereArgs: [id]);
  /// ```
  Future<List<Map<String, Object?>>> query(String table,
      {bool? distinct,
      List<String>? columns,
      String? where,
      List<Object?>? whereArgs,
      String? groupBy,
      String? having,
      String? orderBy,
      int? limit,
      int? offset}) {
    return _db.query(table,
        distinct: distinct,
        columns: columns,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        having: having,
        orderBy: orderBy,
        limit: limit,
        offset: offset);
  }

  /// Executes a raw SQL SELECT query and returns a list
  /// of the rows that were found.
  ///
  /// ```
  /// List<Map> list = await database.rawQuery('SELECT * FROM Test');
  /// ```
  Future<List<Map<String, Object?>>> rawQuery(String sql,
      [List<Object?>? arguments]) {
    return _db.rawQuery(sql, arguments);
  }

  /// Executes a raw SQL UPDATE query and returns
  /// the number of changes made.
  ///
  /// ```
  /// int count = await database.rawUpdate(
  ///   'UPDATE Test SET name = ?, value = ? WHERE name = ?',
  ///   ['updated name', '9876', 'some name']);
  /// ```
  Future<int> rawUpdate(String sql, [List<Object?>? arguments]) {
    return _db.rawUpdate(sql, arguments);
  }

  /// Convenience method for updating rows in the database. Returns
  /// the number of changes made
  ///
  /// Update [table] with [values], a map from column names to new column
  /// values. null is a valid value that will be translated to NULL.
  ///
  /// [where] is the optional WHERE clause to apply when updating.
  /// Passing null will update all rows.
  ///
  /// You may include ?s in the where clause, which will be replaced by the
  /// values from [whereArgs]
  ///
  /// [conflictAlgorithm] (optional) specifies algorithm to use in case of a
  /// conflict. See [ConflictAlgorithm] docs for more details
  ///
  /// ```
  /// int count = await db.update(tableTodo, todo.toMap(),
  ///    where: '$columnId = ?', whereArgs: [todo.id]);
  /// ```
  Future<int> update(String table, Map<String, Object?> values,
      {String? where,
      List<Object?>? whereArgs,
      ConflictAlgorithm? conflictAlgorithm}) {
    return _db.update(table, values,
        where: where,
        whereArgs: whereArgs,
        conflictAlgorithm: conflictAlgorithm);
  }

  /// Executes a raw SQL DELETE query and returns the
  /// number of changes made.
  ///
  /// ```
  /// int count = await database
  ///   .rawDelete('DELETE FROM Test WHERE name = ?', ['another name']);
  /// ```
  Future<int> rawDelete(String sql, [List<Object?>? arguments]) {
    return _db.rawDelete(sql, arguments);
  }

  /// Convenience method for deleting rows in the database.
  ///
  /// Delete from [table]
  ///
  /// [where] is the optional WHERE clause to apply when updating. Passing null
  /// will delete all rows.
  ///
  /// You may include ?s in the where clause, which will be replaced by the
  /// values from [whereArgs]
  ///
  /// Returns the number of rows affected.
  /// ```
  ///  int count = await db.delete(tableTodo, where: 'columnId = ?', whereArgs: [id]);
  /// ```
  Future<int> delete(String table, {String? where, List<Object?>? whereArgs}) {
    return _db.delete(table, where: where, whereArgs: whereArgs);
  }
}
